package it4systems;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.prefs.*;

import javax.swing.*;

public class MarsRescue extends JFrame implements Runnable, KeyEventPostProcessor {
	private MarsPanel marsPanel;
	private MarsMap marsMap;
	private boolean jetLeft = false;
	private boolean jetRight = false;
	private boolean jetBottom = false;
	private JTextArea solutionArea;
	private static final int STOPPED = 0;
	private static final int TEST_SOLUTION = 1;
	private static final int PLAY = 2;
	private int state = STOPPED;
	private int solutionIndex;
	private String solutionText;

	public MarsRescue() {
		super("Dr. Dobb's Mars Rescue Mission Challenge");
		KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventPostProcessor(this);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		marsPanel = new MarsPanel();
		try {
			marsMap = new MarsMap(getClass().getResourceAsStream("/demo.txt"));
		} catch (Exception e) {
			// should not occur, because map.txt is bundled in the jar
			e.printStackTrace();
		}
		marsPanel.setMarsMap(marsMap);
		getContentPane().add(BorderLayout.NORTH, createButtonPanel());
		getContentPane().add(BorderLayout.CENTER, marsPanel);
		solutionArea = new JTextArea();
		solutionArea.append("26446640000000022222202220222220\n");
		solutionArea.append("01131003222222202222000000000000\n");
		solutionArea.setRows(5);
		solutionArea.setBorder(BorderFactory.createTitledBorder("Solution area"));
		getContentPane().add(BorderLayout.SOUTH, solutionArea);
		pack();
		setVisible(true);

		new Thread(this).start();
	}

	private JPanel createButtonPanel() {
		JPanel buttonPanel = new JPanel(new GridLayout(1, 0));

		JButton loadButton = new JButton("Load map");
		loadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					Preferences userPrefs = Preferences.userRoot().node("MarsRescuePreferences");
					String lastFile = userPrefs.get("lastFile", null);

					JFileChooser chooser = new JFileChooser();
					javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter() {
						public boolean accept(File f) {
							return (f.getName().toLowerCase().endsWith(".txt") || f.isDirectory());
						}
						public String getDescription() {
							return "Map text file";
						}
					};
					chooser.setFileFilter(filter);
					if (lastFile != null) {
						chooser.setSelectedFile(new File(lastFile));
					}
					int returnVal = chooser.showOpenDialog(MarsRescue.this);
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						userPrefs.put("lastFile", chooser.getSelectedFile().getAbsolutePath());
						marsMap.load(new FileInputStream(chooser.getSelectedFile()));
						marsPanel.setMarsMap(marsMap);
						marsPanel.revalidate();
						pack();
					}
				} catch (Exception x) {
					JOptionPane.showMessageDialog(
						MarsRescue.this,
						"Error opening file: " + x.getMessage(),
						"Mars Rescue",
						JOptionPane.INFORMATION_MESSAGE);
				}
			}
		});
		buttonPanel.add(loadButton);

		JButton testButton = new JButton("Test solution");
		testButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				solutionText = solutionArea.getText();
				solutionIndex = 0;
				marsMap.reset();
				state = TEST_SOLUTION;
			}
		});
		buttonPanel.add(testButton);

		JButton playButton = new JButton("Play game");
		playButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				marsMap.reset();
				solutionArea.setText("");
				state = PLAY;
			}
		});
		buttonPanel.add(playButton);

		return buttonPanel;
	}

	public void run() {
		while (true) {
			try {
				Thread.sleep(200);
			} catch (InterruptedException e) {
			}
			if (!marsMap.isWon()) {
				if (state == TEST_SOLUTION) {
					while (true) {
						if (solutionIndex < solutionText.length()) {
							int jets = solutionText.charAt(solutionIndex++) - '0';
							if (jets >= 0 && jets <= 7) {
								boolean left = (jets & 1) == 1;
								boolean bottom = (jets & 2) == 2;
								boolean right = (jets & 4) == 4;
								marsMap.update(left, bottom, right);
								break;
							}
						} else
							break;
					}
				} else if (state == PLAY) {
					marsMap.update(jetLeft, jetBottom, jetRight);
					int jets = 0;
					if (jetLeft)
						jets |= 1;
					if (jetBottom)
						jets |= 2;
					if (jetRight)
						jets |= 4;
					solutionArea.append(Character.toString((char) (jets + '0')));
				}
			}
			marsPanel.repaint();
		}
	}

	public boolean postProcessKeyEvent(KeyEvent e) {
		if (e.getID() == KeyEvent.KEY_PRESSED) {
			switch (e.getKeyCode()) {
				case KeyEvent.VK_LEFT :
					jetRight = false;
					jetLeft = true;
					break;
				case KeyEvent.VK_RIGHT :
					jetRight = true;
					jetLeft = false;
					break;
				case KeyEvent.VK_UP :
					jetBottom = true;
					break;
			}
		} else if (e.getID() == KeyEvent.KEY_RELEASED) {
			switch (e.getKeyCode()) {
				case KeyEvent.VK_LEFT :
					jetRight = false;
					jetLeft = false;
					break;
				case KeyEvent.VK_RIGHT :
					jetRight = false;
					jetLeft = false;
					break;
				case KeyEvent.VK_UP :
					jetBottom = false;
					break;
			}
		}
		return false;
	}

	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}
		new MarsRescue();
	}
}
